/** Die Klasse Agent reprsentiert einen Agenten, also eine Ameise oder Biene, oder einen Elefant.<br>
 * Ein solcher Agent weiss, wo er ist, wie seine Zufallszahlen entstehen, wie weit er sehen kann, was er sieht, 
 * er kann ber einen Baum auf sein Genom zugreifen (cf. PatternGenerator) und er weiss, was er gerade macht und 
 * was er macht, wenn mehrere seiner Gene auf seine Nest passen.<br>
 * Ausserdem ist ein solcher Agent immer bemht am Nest zu bleiben.
 *
 * Stand: 09.07.2001
 */

public class Agent{
    
    protected Koord3D position;
    protected Random rand;
    protected Koord3D sichtweite;
    
    protected Blickfeld view;
      
    protected Knoten tree;
    
    protected Action currentAction;
           
    protected int strategy;
    
    
    public Agent(Koord3D koords, Koord3D sichtweite, Blickfeld view, Knoten tree){       
        this.view = view;
	this.tree = tree;
        
        position = koords;

        this.sichtweite = sichtweite;

        rand = new Random((int)(new java.util.Date()).getTime() % Integer.MAX_VALUE );
    }


    public int getGenUsed(){
	if( currentAction != null ){
	    return currentAction.getGenUsed();
	}else{
	    return -1;
	}
    }
    
 
    // gendert am 25.06.2001 -> getActionFromBlickfeld liefert ja jetzt eine LinkedList.
    public void lookAround(){
        try{	    
	    java.util.LinkedList list = tree.getActionFromBlickfeld( view, position );
	    Action[] tempArray = new Action[list.size()];
	    list.toArray( tempArray );	          	    
	    
            switch (strategy){
            case Simulator.FIRST:
                currentAction = tempArray[0];
                break;
            case Simulator.RANDOM:
                currentAction = tempArray[ new java.util.Random().nextInt( tempArray.length ) ];
                break;
            case Simulator.LEAST_JOKER:
                currentAction = tempArray[ getIndexWithLeastJoker( tempArray ) ];
                break;
            case Simulator.MOST_JOKER:
                currentAction = tempArray[ getIndexWithMostJoker( tempArray ) ];
                break;
            default:
                currentAction = null;
            }
        }catch( PatternNotFoundInTreeException e ){
            currentAction = null;
        }
        
    } 
    
    // for debugging purpose:
    // luft die Agent in die richtige Richtung?
    public int getDirectionByGen(){
        try{
            return currentAction.whichWayToMove();
        }catch( NullPointerException e ){
            return -1;
        }
    }
    
    public Koord3D moveYou()
        throws NoMoveLeftException{       
        int givenDirection = getDirectionByGen();    

        int[] listOfMoves = getMovesList();

        int newIndex = 0;
        try{
            newIndex = Math.abs(rand.genrand()) % listOfMoves.length;
        }catch( ArithmeticException e ){
            // falls 0 Zge mglich sind
            throw new NoMoveLeftException();
        }
       
        if( givenDirection >= 0 ){
            // hier wird geguckt, ob in die vorgegebene Richtung berhaupt gegangen werden kann
            if( contained( givenDirection, listOfMoves ) ){
                position = view.translate( givenDirection, position );
            }else{
                position = view.translate( listOfMoves[ Math.abs(newIndex) ], position );
            }
        }else{
            position = view.translate( listOfMoves[ Math.abs(newIndex) ], position );
        }

        return position;
    }

    // 28.06.2001 wird von Nest.setPosAgent() verwendet
    public void setPosition( Koord3D newPosition ){
	position = newPosition;
    }

    // wird z.B. in Nest.runOneRound()
    public Koord3D getPosition(){
	return position;
    }

    protected boolean contained( int direction, int[] list ){
        for( int i = 0; i < list.length; i++ ){
            if( list[i] == direction )
                return true;
        }

        return false;
    }

   
    protected int[] getMovesList(){
        int[] result = new int[ view.size() ];

        int index = 0;
        for( int i = 0; i < view.size(); i++ ){
            if( view.getStateAt( i, position ) == Nest.LEER ){
                result[ index ] = i;
                index++;
            }
        }

	int[] tempArray = new int[ index ];
	int index2 = 0;
	for( int i = 0; i < index; i++){
	    if(hasAtLeastOneNeighboringStone(view.translate( result[i], position ))){
		tempArray[index2]=result[i];
		index2++;
	    }		
	}

	// falls gar kein Stein sichtbar ist, soll der Agent doch zufllig irgendwohin gehen
	if( index2 == 0 ){
	    tempArray = result;
	    index2 = index;
	}

        // erzeuge ein neues Array, in dem es keine leeren Eintrge gibt
        int[] final_result = new int[ index2 ];
        for( int i = 0; i<index2; i++){
            final_result[i] = tempArray[i];
        }

        return final_result;
    }

    // diese Funktion kontrolliert ob, wenn der Agent in ein Richtung gehen wrde, er dort in Sichtfeld kommt, wo er noch einen Stein sieht.
    protected boolean hasAtLeastOneNeighboringStone( Koord3D virtualPos ){
        
        int index = 0;
        while( index < view.size() && view.getStateAt( index, virtualPos ) != Nest.STEIN ){
	    index++;
	}

        return index <  view.size();
    }
   

    // 12.06.: Liefert den Index des Eintrags mit den wenigsten Jokern.
    // gibt es mehrere solche, so wird der erste genommen.
    protected int getIndexWithLeastJoker( Action[] temp ){
        int index = 0;
        int value = temp[0].getNumberOfJoker();

        for( int i = 1; i<temp.length; i++){
            if( temp[i].getNumberOfJoker() < value ){
                index = i;
                value = temp[i].getNumberOfJoker();
            }
        }

        return index;
    }

    // dasselbe wie oben in grn
    protected int getIndexWithMostJoker( Action[] temp ){
        int index = 0;
        int value = temp[0].getNumberOfJoker();

        for( int i = 1; i<temp.length; i++){
            if( temp[i].getNumberOfJoker() > value ){
                index = i;
                value = temp[i].getNumberOfJoker();
            }
        }

        return index;
    }

    public boolean bOrNot2B(){
        try{
            return currentAction.shallBuild();
        }catch( NullPointerException e ){
            return false;
        }
    }
    
    // 12.06.
    public void setStrategy( int strategy ){
        this.strategy = strategy;
    }

    public int getStrategy(){
        return strategy;
    }  
}
