import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
/**
 * Title: Bee3DModell
 * Description: This Class is for representing bees in our 3D modell of the hive.
 * @author: Dimitri Missoh & Daniel Schlosser
 * @version 1.0
 */

public class Bee3DModell extends BranchGroup
{
      private Vector3d vec;
      private Transform3D transf3D;
      private TransformGroup translation;
      private Sphere bee;
      private Appearance beeAppearance;
      private ColoringAttributes beeColorA;
      private Material beeMaterial;

    /**
     * Creates e bee at the given point
     * @param x the x coordinate
     * @param y the y coordinate
     * @param z the z coordinate
     */
    public Bee3DModell(int x, int y, int z) {
           super();
           vec=new Vector3d();
           vec.set(x,y,z);
           transf3D=new Transform3D();
           transf3D.setTranslation(vec);
           translation=new TransformGroup(transf3D);
           translation.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
           beeAppearance=new Appearance();
           beeColorA=new ColoringAttributes(new Color3f(java.awt.Color.red),
                                            ColoringAttributes.SHADE_GOURAUD);
           beeAppearance.setColoringAttributes(beeColorA);
           beeMaterial=new Material();
           beeMaterial.setAmbientColor(1.0f,0.0f,0.0f);
           beeMaterial.setDiffuseColor(1.0f,0.0f,0.0f);
           beeMaterial.setSpecularColor(1.0f,1.0f,1.0f);
           beeMaterial.setShininess(100.0f);
           beeMaterial.setLightingEnable(true);
           beeAppearance.setMaterial(beeMaterial);
           bee=new Sphere(0.5f, Primitive.GENERATE_NORMALS |
		 Primitive.ENABLE_APPEARANCE_MODIFY |Primitive.GENERATE_NORMALS_INWARD
		 ,beeAppearance);
           translation.addChild(bee);
           this.setCapability(BranchGroup.ALLOW_DETACH);
           this.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
           this.setCapability(BranchGroup.ENABLE_PICK_REPORTING);
           this.addChild(translation);
    }

    /**
     * Moves the bee from current position to next
     * @param pos The coordinate of the next position
     */
    public void move(Koord3D pos)
    {
	if(pos==null)
	    return;
        vec.set(pos.getX(),pos.getY(),pos.getZ());
        transf3D.setTranslation(vec);
        translation.setTransform(transf3D);
    }
}
